# -*- coding: utf-8 -*-


import tornado.options

from .engine import Engine as dpEngine

session_default_expire_in = tornado.options.options.session_exp_in or 7200


class Controller(dpEngine):
    def __init__(self, application, request, **kwargs):
        super(Controller, self).__init__()

        self.prefix = kwargs.get('prefix')
        self.parent = kwargs.get('parent')

        self._write = []
        self._finish = None
        self._render = None

        self.head_requested = False
        self.post_requested = False
        self.get_requested = False
        self.patch_requested = False
        self.delete_requested = False
        self.put_requested = False

    def initialize(self, prefix, parent=None):
        self.prefix = prefix
        self.parent = parent

    @property
    def request(self):
        return self.parent.request

    def get_argument(self, name, default=None, strip=True, cast=None, fmt=None):
        ret = self.parent.get_argument(name, default, strip)

        try:
            if ret and cast == int and fmt == 'yyyymmdd':
                try:

                    return self.helper.datetime.time(
                        self.helper.datetime.datetime(
                            yyyymmdd=self.helper.numeric.extract_numbers(ret or '')))

                except:
                    return False

            elif ret and cast == int:
                return int(ret)
            elif ret and cast == long:
                return long(ret)
            elif ret and cast == float:
                return float(ret)

        except ValueError:
            return default

        return ret

    def get_user_agent(self, parsed=True):
        return self.parent.get_user_agent(parsed=parsed)

    @property
    def remote_ip(self):
        return self.parent.remote_ip

    def set_secure_cookie(self, name, value, expires_days=30, version=2, **kwargs):
        return self.parent.set_secure_cookie(name, value, expires_days=expires_days, version=version, **kwargs)

    def get_secure_cookie(self, name, value=None, max_age_days=31, min_version=None):
        return self.parent.get_secure_cookie(name, value=value, max_age_days=max_age_days, min_version=min_version)

    def get_sessionid(self):
        return self.parent.get_sessionid()

    def set_sessionid(self, sessionid=None):
        return self.parent.set_sessionid(sessionid=sessionid)

    def get_session_value(self, name, expire_in=None, sessionid=None):
        return self.parent.get_session_value(name, expire_in=expire_in, sessionid=sessionid)

    def get_session_value_ttl(self, name):
        return self.parent.get_session_value_ttl(name)

    def set_session_value(self, name, value, expire_in=session_default_expire_in):
        return self.parent.set_session_value(name, value, expire_in=expire_in)

    def session(self, name, value=None, expire_in=session_default_expire_in):
        return self.parent.session(name, value=value, expire_in=expire_in)

    def _prefix(self, url):
        if 'X-Proxy-Prefix' in self.request.headers:
            prefix = self.request.headers['X-Proxy-Prefix']
            prefix = prefix[:-1] if prefix.endswith('/') else prefix

            if url.startswith(prefix):
                url = url[len(prefix):] or '/'

        return url

    def redirect(self, url, prefix=False, permanent=False, status=None):
        if hasattr(self.parent, '__postprocessed__'):
            return

        setattr(self.parent, '__postprocessed__', True)

        if prefix:
            url = self._prefix(url)

        try:
            self.parent.redirect(url, permanent, status)
        except (AttributeError, TypeError):
            pass

    def render(self, template_name, kwargs=None):
        self._render = {'t': template_name, 'k': kwargs}

    def render_string(self, template_name, kwargs=None):
        if kwargs:
            return self.view.render_string(self, template_name, kwargs)
        else:
            return self.view.render_string(self, template_name)

    def write(self, chunk):
        self._write.append(chunk)

    def finish(self, chunk=None):
        self._finish = chunk



# -*- coding: utf-8 -*-


class _Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(_Singleton, cls).__call__(*args, **kwargs)

        return cls._instances[cls]


class Singleton(_Singleton('SingletonMeta', (object,), {})):
    pass


# -*- coding: utf-8 -*-


from __future__ import absolute_import
from dp_tornado.engine.helper import Helper as dpHelper

import smtplib


class SmtpSender(object):
    def __init__(self, e, host=None, port=None, userid=None, password=None, ehlo=None, tls=False):
        self.e = e
        self.host = host
        self.port = port
        self.userid = userid
        self.password = password
        self.ehlo = ehlo
        self.tls = tls
        self.connected = False
        self.connection = None

    def connect(self):
        self.connection = smtplib.SMTP(self.host, self.port)

        if self.userid and self.password:
            self.connection.login(self.userid, self.password)

            if self.ehlo is not None:
                self.connection.ehlo(self.ehlo)

            if self.tls:
                self.connection.starttls()

        self.connected = True

    def send(self, subject, content, from_user, to_user, html=True):
        if self.connected:
            from email.mime.text import MIMEText

            msg = MIMEText('%s\n' % content, 'html' if html else 'plain', 'UTF-8')
            msg['Subject'] = subject
            msg['From'] = from_user
            msg['To'] = to_user

            if html:
                msg['Content-Type'] = 'text/html; charset=utf-8'

            if self.e.helper.system.py_version <= 2:
                self.connection.sendmail(from_user, to_user, msg.as_string())
            else:
                self.connection.send_message(msg)

        return False

    def quit(self):
        if self.connected:
            self.connection.quit()

        return False


class SmtpHelper(dpHelper):
    def send(self, to_user, subject, content, from_user=None, cc=None, attach=None,
             host=None, port=None, userid=None, password=None, ehlo=None, tls=False, html=True):
        s = SmtpSender(e=self, host=host, port=port, userid=userid, password=password, ehlo=ehlo, tls=tls)
        s.connect()
        s.send(subject=subject, content=content, from_user=from_user, to_user=to_user, html=html)
        s.quit()

        return True



# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper
from dp_tornado.engine.engine import Engine as dpEngine


_s3_connection_ = None


def s3_connection():
    global _s3_connection_

    if _s3_connection_:
        return _s3_connection_

    try:
        from boto.s3.connection import S3Connection

        import logging
        logging.getLogger('boto').setLevel(logging.CRITICAL)

        _s3_connection_ = S3Connection

    except ImportError as e:
        raise e

    return _s3_connection_


class S3Bridge(dpEngine):
    def __init__(self, public, secret):
        try:
            from boto.s3.connection import S3Connection

            import logging
            logging.getLogger('boto').setLevel(logging.CRITICAL)

        except ImportError as e:
            raise e

        self.conn = S3Connection(public, secret)

    def bucket(self, bucket_name):
        return self.conn.get_bucket(bucket_name)

    def set_contents_from_file(self, bucket_name, key, fp, url=False):
        try:
            from boto.s3.key import Key
        except ImportError as e:
            raise e

        fp.seek(0)
        bucket = self.bucket(bucket_name)
        obj = Key(bucket)
        obj.key = key
        res = obj.set_contents_from_file(fp)

        if url:
            return obj.generate_url(0)
        else:
            return res

    def copy(self, from_bucket_name, from_key_name, to_bucket_name, to_key_name):
        try:
            from boto.s3.key import Key
        except ImportError as e:
            raise e

        from_bucket = self.bucket(from_bucket_name)
        from_key = Key(from_bucket)
        from_key.key = from_key_name

        to_bucket = self.bucket(to_bucket_name)
        return from_key.copy(to_bucket, to_key_name)


class S3Helper(dpHelper):
    def connect(self, public, secret):
        return S3Bridge(public, secret)

    def prepare_post(self,
                     aws_access_key_id,
                     aws_secret_access_key,
                     bucket_name,
                     key,
                     success_action_redirect=None,
                     max_content_length=None,
                     expires_in=6000,
                     acl=None):
        s3 = s3_connection()(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        if not success_action_redirect:
            fields = [{"name": "success_action_status", "value": "201"}]
            conditions = ['{"success_action_status": "201"}']
        else:
            fields = [{"name": "success_action_redirect", "value": success_action_redirect}]
            conditions = ['{"success_action_redirect": "%s"}' % success_action_redirect]

        payload = s3.build_post_form_args(
            bucket_name=bucket_name,
            key=key,
            expires_in=expires_in,
            acl=acl,
            max_content_length=max_content_length,
            fields=fields,
            conditions=conditions)

        return payload
